// Copyright (C) 1997, 1998 by DI Wolfgang Platzer, IAIK
// email: wplatzer@iaik.tu-graz.ac.at
//
// All rights reserved.
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions
// are met:
// 1. Redistributions of source code must retain the above copyright
//    notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
//    notice, this list of conditions and the following disclaimer in the
//    documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
// FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
// DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
// OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
// HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
// LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
// OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
// SUCH DAMAGE.
//

package org.w3c.jigsaw.http.socket;

import java.net.*;
import java.security.cert.*;

import iaik.security.ssl.*;

import org.w3c.jigsaw.http.httpd;

/**
 * This class adds SSL features to a SocketClient. 
 * You can get the following information:
 * <ul>
 * <li>uses this Socket a SSL connection
 * <li>get the active cipher suite 
 * <li>get the active compression method
 * <li>get the certificate chain sent by the client
 * </ul>
 */
public class SSLSocketClient extends SocketClient {

  /**
   * Indicates wheter this SocketClient is bind to a SSL connection.
   */
  protected boolean ssl_connection = false;

  /**
   * Creates a new SSLSocketClient. Simply calls the constructor of its superclass.
   */
  public SSLSocketClient(httpd server, SocketClientFactory pool, SocketClientState state) {
    super(server, pool, state);
  }

  /**
   * Bind this SocketClient to a Socket and check if this is
   * a SSL connection.
   */
  public void bind(Socket socket) {
    ssl_connection = (socket instanceof SSLSocket);

// DEBUG
//    if (ssl_connection)
//      ((SSLSocket)socket).setDebugStream(System.out);
      
    try {
      socket.setSoTimeout(server.getRequestTimeOut());
    } catch (SocketException ex) {
      server.errlog("Unable to set socket timeout!");
    }
    
    super.bind(socket);
  }
  
  /**
   * Returns true if this is a SSL connection.
   *
   * @return true if this is a SSL connection
   */
  public boolean isSSLConnection() {
    return ssl_connection;
  }

  /**
   * Returns the certificate chain of the client or null if the 
   * client didn't send a certificate chain.
   *
   * @return the client certificate chain
   */
  public X509Certificate[] getClientCertificateChain() {
    if (ssl_connection)
      return ((SSLSocket)socket).getPeerCertificateChain();
    else
      return null;
  }

  /**
   * Returns the active cipher suite or null if this is not a
   * SSL connection.
   *
   * @return the active cipher suite
   */
  public CipherSuite getActiveCipherSuite() {
    if (ssl_connection)
      return ((SSLSocket)socket).getActiveCipherSuite();
    else
      return null;
  }

  /**
   * Returns all cipher suites supported by the client or null 
   * if this is not a SSL connection.
   *
   * @return all cipher suites supported by the client
   */
  public CipherSuite[] getSupportedCipherSuites() {
    if (ssl_connection)
      return ((SSLSocket)socket).getClientSupportedCipherSuites();
    else
      return null;
  }

  /**
   * Returns the active compression method or null if this is not a
   * SSL connection.
   *
   * @return the active compression method
   */
  public CompressionMethod getActiveCompressionMethod() {
    if (ssl_connection)
      return ((SSLSocket)socket).getActiveCompressionMethod();
    else
      return null;
  }

  /**
   * Returns all compression methods supported by the client or null 
   * if this is not a SSL connection.
   *
   * @return all compression methods supported by the client
   */
  public CompressionMethod[] getSupportedCompressionMethods() {
    if (ssl_connection)
      return ((SSLSocket)socket).getClientSupportedCompressionMethods();
    else
      return null;
  }
}